#include "Juego.h"
#include <iostream>
#include <string>
#include <sstream>
#include <list>

#include "Graficos.h"
#include "Input.h"
#include "GeneradorAleatorios.h"
#include "GestorRanking.h"

using namespace std;

int Juego::estado = 1;
bool Juego::fin = false;
int Juego::difficulty = 1;
int Juego::tablero[39][22];	
int Juego::manPosX = 10;
int Juego::manPosY = 21;
int Juego::maxScore = 200;
int Juego::score = 0;
int Juego::pasos = 0;
int Juego::tiempoPantalla = 0;
int Juego::tiempoRestante = 0;
bool Juego::muerte = false;
int Juego::cntMuerte = 0;
string Juego::name = "AAA";
int Juego::pos = 0;
int Juego::cntCambio = 0;

list<Puntuacion> Juego::ranking;

Juego::Juego()
{
	
}

void Juego::empezar()
{
	struct timeval time;	

	Graficos::init();
	Input::init();
	GeneradorAleatorios::init();
	list<Puntuacion> ranking;
	GestorRanking::getRanking(ranking);

	int uSecNow;
	int secNow;
	int uSecLast;
	int secLast;

	tiempoPantalla = 0;
	
	gettimeofday(&time, NULL);
	secLast = time.tv_sec;
	uSecLast = time.tv_usec;
	tiempoPantalla = time.tv_sec;

	estado = 1;	//1 menu, 2 Juego, 3 Ranking, 4 Meter nombre, 5 cambio pantalla

	// Esperamos hasta que el usuario pulse Escape para salir
    while (!fin)
	{
		Input::update();

		if (estado==1)
		{
			if (Input::spacePressed())
			{
				if (Graficos::getOption()==1)
				{
					estado = 5;
					cntCambio=0;
					resetTablero();
					//poner numero de mina
					Graficos::setMinesAround(getMinesAround());
				}				
				else
				{
					ranking.clear();
					GestorRanking::getRanking(ranking);
					estado = 3;
				}
			}
			if (Input::downPressed())
			{
				Graficos::setOption(2);
			}
			if (Input::upPressed())
			{
				Graficos::setOption(1);
			}
			if (Input::escapePressed())
			{
				fin = true;
			}
		}
		else if (estado==2)
		{
			if (!muerte)
				moverPersonaje();
			if (Input::escapePressed())
			{
				estado = 1;
			}
		}
		else if (estado==3)
		{
			if (Input::escapePressed())
			{
				estado = 1;
			}
		}
		else if (estado==4)
		{
			meterNombre();
		}
		else if (estado==5)
		{
			
		}

		gettimeofday(&time, NULL);
		secNow = time.tv_sec;
		uSecNow = time.tv_usec;

        if (uSecNow + secNow*1000000 -
			uSecLast - secLast*1000000 > 1000000.0f / RATE) 
		{	
			uSecLast = uSecNow;
			secLast = secNow;
			if (estado==1)
			{
				Graficos::updateMenu();	
			}
			else if (estado==2)
			{
				
				
				if (!muerte)
				{
					score = maxScore - (time.tv_sec-tiempoPantalla) - pasos;
					if (score<0)	score=0;
					tiempoRestante = time.tv_sec-tiempoPantalla;
					Graficos::updateGame(tablero, difficulty, tiempoRestante , score, false);
				}
				else
				{
					Graficos::updateGame(tablero, difficulty, tiempoRestante, score, true);
					cntMuerte++;
					if (cntMuerte>=90)
					{
						//Cambiar al estado de ranking
						estado=4;
						cntMuerte=0;
						muerte=false;
					}
				}
			}	
			else if (estado==3)
			{
				Graficos::updateRanking(ranking);
			}
			else if (estado==4)
			{
				Graficos::updateName(name,pos);
			}
			else if (estado==5)
			{
				Graficos::updateCambio(difficulty, cntCambio);

				cntCambio++;
			
				if (cntCambio>45)
				{
					cntCambio=0;
					estado = 2;
					struct timeval time;	
					gettimeofday(&time, NULL);
					tiempoPantalla = time.tv_sec;
				}
			}				
		}
    }

    Graficos::end();
}

void Juego::resetTablero()
{
	manPosX = 20;
	manPosY = 21;

	score = 0;

	Graficos::setManPos(manPosX,manPosY);
	
	for (int i=0; i<39; i++)
	{
		for (int j=0; j<22; j++)
		{
			tablero[i][j]=0;	//0 sin pisar, 1 pisado, 2 mina
		}
	}
	tablero[manPosX][manPosY] = 1;

	//1 meter minas según dificultad
	//Las metemos de forma equidistante y después las "agitamos"
	int sepRow = 8 - difficulty;
	int sepColumn = 8 - difficulty;

	if (sepRow < 2)	sepRow=2;
	if (sepColumn < 2) sepColumn=2;
	
	int randI,randJ;
	for (int i=0; i<39; i+=sepColumn)
	{
		for (int j=0; j<22; j+=sepRow)
		{
			randI = i + GeneradorAleatorios::getInt(-3,3);
			randJ = j + GeneradorAleatorios::getInt(-2,2);
			if (randI<0)	randI=0;
			if (randI>38)	randI=38;
			if (randJ<0)	randJ=0;
			if (randJ>21)	randJ=21;
			if (!(randI==manPosX) || !(randJ==manPosY))
				tablero[randI][randJ]=2;	
		}
	}

	
}

void Juego::moverPersonaje()
{
	bool posUpdated=false;
	if (Input::upPressed())
	{
		manPosY--;
		posUpdated=true;

		pasos++;
	}
	else if (Input::downPressed())
	{
		if (manPosY<21)
		{
			manPosY++;
			posUpdated=true;
			
			pasos++;
		}
	}
	else if (Input::rightPressed())
	{
		if (manPosX<38)
		{
			manPosX++;
			posUpdated=true;
			
			pasos++;
		}
	}
	else if (Input::leftPressed())
	{
		if (manPosX>0)
		{
			manPosX--;
			posUpdated=true;
			
			pasos++;
		}
	}
	
	if (posUpdated)
	{
		
		if (manPosY == -1)
		{
			pasarPantalla();
		}
		else if (tablero[manPosX][manPosY] == 2)
		{
			muerte=true;
		}
		else
		{
			//marcamos el camino
			tablero[manPosX][manPosY] = 1;

			//poner numero de mina
			Graficos::setMinesAround(getMinesAround());

			//colocar hombre
			Graficos::setManPos(manPosX,manPosY);
		}
	}
}

int Juego::getMinesAround()
{
	int x;
	int y;
	int mines=0;

	//arriba izquierda
	x = manPosX-1;
	y = manPosY-1;
	if (x>=0 && y>=0 && tablero[x][y]==2)
		mines++;

	//arriba centro
	x = manPosX;
	y = manPosY-1;
	if (y>=0 && tablero[x][y]==2)
		mines++;

	//arriba derecha
	x = manPosX+1;
	y = manPosY-1;
	if (x<38 && y>=0 && tablero[x][y]==2)
		mines++;

	//centro izquierda
	x = manPosX-1;
	y = manPosY;
	if (x>0 && tablero[x][y]==2)
		mines++;

	//centro derecha
	x = manPosX+1;
	y = manPosY;
	if (x<38 && tablero[x][y]==2)
		mines++;

	//abajo izquierda
	x = manPosX-1;
	y = manPosY+1;
	if (x>=0 && y<22 && tablero[x][y]==2)
		mines++;

	//abajo centro
	x = manPosX;
	y = manPosY+1;
	if (y<22 && tablero[x][y]==2)
		mines++;

	//abajo derecha
	x = manPosX+1;
	y = manPosY+1;
	if (x<38 && y<22 && tablero[x][y]==2)
		mines++;

	return mines;
}

void Juego::pasarPantalla()
{
	difficulty ++;
	maxScore += score;
	pasos = 0;
	resetTablero();

	estado = 5;
}

void Juego::meterNombre()
{
	if (Input::downPressed())
	{
		if (int(name[pos]) < 122)
			name[pos]+=1;
	}
	if (Input::upPressed())
	{
		if (int(name[pos]) > 48)
			name[pos]-=1;
	}
	if (Input::leftPressed())
	{
		if (pos>0)
		{
			pos--;
		}
	}
	if (Input::rightPressed())
	{
		if (pos<2)
		{
			pos++;
		}
	}
	if (Input::spacePressed())
	{
		meterPuntuacion();
		estado = 1;
		name="AAA";

		maxScore = 200;
		score=0;
		pasos = 0;
		
		struct timeval time;	
		gettimeofday(&time, NULL);
		tiempoPantalla = time.tv_sec;
	}
}

void Juego::meterPuntuacion()
{
	Puntuacion puntuacion;
	puntuacion.nombre = name;
	puntuacion.puntos = score;

	ranking.clear();
	list<Puntuacion>::iterator it;
	GestorRanking::getRanking(ranking);
	it=ranking.end();
	it--;	

	if (puntuacion.puntos>(*it).puntos)
	{
		GestorRanking::setRanking(ranking,puntuacion);
	}	
}
